import os
import json
import time

import requests
import urllib3
from bs4 import BeautifulSoup
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
import random

urllib3.disable_warnings()


class WebC(object):
    def __init__(self, url=None, headers=None, cookies=None, proxies=None, timeout=30, ip=False):
        _headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/53.0.2785.143 Safari/537.36'
        }
        self._url = url
        self._save = False
        self._encode = 'utf-8'
        self._response = None
        self._timeout = timeout
        self._headers = headers
        self._proxies = proxies
        self._cookies = cookies
        if not headers:
            self.headers = _headers
        if not proxies and ip:
            self._proxies = self.random_ip
        if cookies:
            self.cookies = cookies

    @property
    def save(self):
        return self._save

    @save.setter
    def save(self, value):
        self._save = value

    @property
    def encode(self):
        return self._encode

    @encode.setter
    def encode(self, char):
        self._encode = char

    @property
    def headers(self):
        return self._headers

    @headers.setter
    def headers(self, header):
        self._headers = header

    @property
    def cookies(self):
        return self._cookies

    @cookies.setter
    def cookies(self, cookie_str):
        cookies_list = cookie_str.split(';')
        cookie_dict = {}
        for i in cookies_list:
            key, val = i.split('=', 1)
            cookie_dict[key] = val
        self._cookies = cookie_dict

    @property
    def session(self):
        return requests.session()

    @property
    def response(self):
        self._response = requests.get(self._url, headers=self._headers, cookies=self._cookies, timeout=self._timeout,
                                      proxies=self._proxies)
        return self._response

    @property
    def html(self, save_path=None):
        resp = self.response
        resp.encoding = self.encode
        html = resp.text
        if self.save:
            with open('{}.html'.format(save_path), 'w') as f:
                f.write(html)
        return html

    @property
    def soup(self):
        if os.path.exists('result.html'):
            soup = BeautifulSoup(open('result.html'), 'html.parser')
        else:
            soup = BeautifulSoup(self.response.text, 'html.parser')
        return soup

    @property
    def selenium_driver(self):
        # setting
        chrome_options = Options()
        chrome_options.add_argument('--headless')
        chrome_options.add_argument('--disable-gpu')
        desired_capabilities = DesiredCapabilities.CHROME  # 修改页面加载策略
        desired_capabilities["pageLoadStrategy"] = "none"  # 注释这两行会导致最后输出结果的延迟，即等待页面加载完成再输出
        driver = webdriver.Chrome(chrome_options=chrome_options)
        # driver.implicitly_wait(10)
        driver.get(self._url)
        if self._cookies:
            driver.add_cookie(self._cookies)
        return driver

    def get_json_dict(self):
        return json.loads(self.response.content)

    def get_payload_data(self, data):
        resp = requests.post(self._url, json=data, headers=self._headers, proxies=self._proxies)
        ct = resp.content
        json_dict = json.loads(ct)
        return json_dict

    @property
    def random_ip(self):
        time.sleep(0.5)
        url = 'https://www.xicidaili.com/nn/'
        web_data = requests.get(url, headers=self._headers)
        soup = BeautifulSoup(web_data.text, 'html.parser')
        ips = soup.find_all('tr')
        ip_list = []
        for i in range(1, len(ips)):
            ip_info = ips[i]
            tds = ip_info.find_all('td')
            ip_list.append(tds[1].text + ':' + tds[2].text)

        proxy_list = []
        for ip in ip_list:
            proxy_list.append('http://' + ip)
        proxy_ip = random.choice(proxy_list)
        proxies = {'http': proxy_ip}
        return proxies



from tools.timer import Time
import traceback
import numpy
import sys
import os
from threading import get_ident, current_thread, Lock
from functools import wraps


# import inspect

# 获取被调用函数所在模块文件名
# print(inspect.stack()[1][1])
# print(sys._getframe(1).f_code.co_filename)

# 获取被调用函数名称
# print(inspect.stack()[1][3])
# print(sys._getframe(1).f_code.co_name)

# 获取被调用函数在被调用时所处代码行数
# print(inspect.stack()[1][2])
# print(sys._getframe(1).f_lineno)


class Logger(object):
    def __init__(self, level=0, full=None):
        self.level = level
        self.mode = ['\033[1;32mINFO\033[0m', '\033[1;33mWARNING\033[0m', '\033[1;31mERROR\033[0m']
        self.time = Time()
        self.lock = Lock()
        self.time.set_fmt('%H:%M:%S')
        self.fmt = '\r[{}] [{}] [{}] [{}]: {}'
        # self.line_num = None
        self.pre_module = None
        self.pre_func = sys._getframe(1).f_code.co_name

    def sp(self, *msg):
        idx = 0
        for i in msg:
            _type = str(type(i)).strip('<class ').strip('>')
            if isinstance(i, list):
                _len = len(i)
                print('{} {}'.format('-' * 100, idx))
                print('Type:{}  Lenght:{}'.format(_type, _len))
                i_len = _len // 10
                index = [i * 10 for i in range(i_len)]
                for j in index:
                    print(i[j:j + 10])
            elif isinstance(i, numpy.ndarray):
                print('{} {}'.format('-' * 100, idx))
                _shape = i.shape
                print('Type:{}  Shape:{}'.format(_type, _shape))
                print(i)
            elif isinstance(i, int):
                print('{} {}'.format('-' * 100, idx))
                print(i)
            else:
                print('{} {}'.format('-' * 100, idx))
                # print('Type:{}  Lenght:{}   Words:{}'.format(_type, len(i), len(i.split(' '))))
                print(i)
            idx = idx + 1

    @property
    def prompt(self):
        # self.line_num = sys._getframe(1).f_lineno
        pre_module = sys._getframe(2).f_code.co_filename.split('/')[-1].split('.')[0]
        time = self.time.get_fmt_time
        thread_name = current_thread().name
        if thread_name == 'MainThread':
            self.pre_func = ''
        index = '{}{}'.format(pre_module, self.pre_func)
        return time, thread_name, index

    def info(self, msg, end='\n'):
        level = 1
        if level >= self.level:
            mode = self.mode[level - 1]
            prompt = self.prompt
            print(self.fmt.format(prompt[0], prompt[1], prompt[2], mode, msg), end=end)

    def warning(self, msg):
        level = 2
        if level >= self.level:
            mode = self.mode[level - 1]
            prompt = self.prompt
            print(self.fmt.format(prompt[0], prompt[1], prompt[2], mode, msg))

    def error(self, msg=None):
        level = 3
        if level >= self.level:
            mode = self.mode[level - 1]
            prompt = self.prompt
            type_, value_, traceback_ = sys.exc_info()
            ex = traceback.format_exception(type_, value_, traceback_)
            print(self.fmt.format(prompt[0], prompt[1], prompt[2], mode, '\033[1;31m{}\033[0m'.format('*' * 100)))
            for i in ex[1:]:
                print(self.fmt.format(prompt[0], prompt[1], prompt[2], mode, i.splitlines(True)[0].strip()))

            if msg:
                print(self.fmt.format(prompt[0], prompt[1], prompt[2], mode, msg))
            print(self.fmt.format(prompt[0], prompt[1], prompt[2], mode, '\033[1;31m{}\033[0m'.format('*' * 100)))

    def log(self, level=0):
        def L(func):
            @wraps(func)
            def wrapper(*args, **kwargs):
                self.pre_func = '|' + func.__qualname__
                return func(*args, **kwargs)

            return wrapper

        return L



from openpyxl import Workbook
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from tools.logger import Logger
import xlrd
import xlsxwriter

lg = Logger()


class Sql(object):

    def __init__(self, user, password, port, dbname, char, base):
        self.engine = create_engine(
            'mysql://{}:{}@localhost:{}/{}?charset={}'.format(user, password, port, dbname, char),
            echo=False)
        self.__Base = base
        self.__Base.metadata.create_all(self.engine)
        self.__Session = sessionmaker(bind=self.engine)
        self.__session = self.__Session()

    def write(self, msg):
        self.__session.add(msg)
        self.__session.commit()

    @property
    def session(self):
        return self.__session


class ExcelWriter(object):
    def __init__(self, path):
        self.work_book = xlsxwriter.Workbook(path)  # 创建一个excel文件
        self.sheet_list = []

    def create_sheet(self, sheet_name='sheet'):
        self.sheet_list.append(self.work_book.add_worksheet(sheet_name))  # 在文件中创建一个名为TEST的sheet,不加名字默认为sheet1

    def write_line(self, line, row_num=0, col_num=0, sheet_index=0):
        self.sheet_list[sheet_index].write_row(row_num, col_num, line)

    def write_lines(self, lines, sheet_index=0):
        if not self.sheet_list:
            self.create_sheet()
        for i, d in enumerate(lines):
            self.sheet_list[sheet_index].write_row(i, 0, d)

    def write_cols(self, *args, sheet_index=0):
        col_lenght = len(args)
        tag = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
        tag_list = list(tag[:col_lenght])
        for i, d in enumerate(args):
            self.sheet_list[sheet_index].write_column('{}{}'.format(tag_list[i], 2), d)

    def close_book(self):
        self.work_book.close()


class ExcelReader(object):
    def __init__(self, path=None):
        self.sheet_list = []
        self.name = path.split('/')[-1].split('.')[0]
        self.data = []
        if path:
            self.read_excel(path)

    def read_excel(self, path):
        wb = xlrd.open_workbook(filename=path)  # 打开文件
        tb_list = wb.sheet_names()  # 获取所有表格名字
        wb_num = len(tb_list)

        # get sheet object list
        for i in range(wb_num):
            sheet = wb.sheet_by_index(i)  # 通过索引获取表格
            self.sheet_list.append(sheet)

        # get data
        for i in self.sheet_list:
            table_data = []
            tabel_length = len(i.col_values(0))
            for j in range(tabel_length):
                table_data.append(i.row_values(j))  # 获取行内容
            self.data.append(table_data)

        return self.data

    def to_csv(self, path=None, sheet_index=0):
        if not path:
            path = self.name + '.csv'
        csv = CsvWriter(path)
        csv.write_lines(self.data[sheet_index])


class CsvWriter(object):

    def __init__(self, path=None):
        self.save_path = path
        self.name = self.save_path.split('/')[-1].split('.')[0]

    def set_save_path(self, path):
        self.save_path = save_path

    def write_line(self, line):
        # ['1','2','3']
        with open(self.save_path, 'a+') as f:
            line = [str(i) for i in line]
            f.write(','.join(line) + '\n')

    @lg.log()
    def write_lines(self, lines):
        # [['1','2','3'], [...], ...]
        with open(self.save_path, 'a+') as f:
            for i in lines:
                line = ','.join([str(j) for j in i])
                f.write(line + '\n')
                f.flush()
            lg.info('File \'{}\' Write Done!'.format(self.save_path))

    @lg.log()
    def write_cols(self, *args):
        # [['headline','1','2','3'], [...], ...]
        with open(self.save_path, 'a+') as f:
            for i in zip(*args):
                line = ','.join(i) + '\n'
                f.write(line)
                f.flush()
            lg.info('Write Done!')


class CsvReader(object):
    def __init__(self, path=None):
        self.data = []
        self.name = path.split('/')[-1].split('.')[0]
        if path:
            self.read_csv(path)

    def read_csv(self, path):
        with open(path, 'r') as f:
            for i in f:
                self.data.append(i.strip())

        return self.data

    @lg.log()
    def to_execl(self, path=None):
        if not path:
            path = self.name + '.xlsx'
        ew = ExcelWriter(path)
        lines = [i.split(',') for i in self.data]
        ew.write_lines(lines)
        ew.close_book()
        lg.info('File \'{}\' Write Done!'.format(path))




from setuptools import find_packages, setup

with open ( "README.md" , "r" ) as fh :
    long_description = fh . read ()

setup(
    name='tfuc',
    version='1.0.9',
    author = "dongkai",
    author_email = "wdkany@qq.com",
    description = "some useful tools",
    long_description = long_description ,
    long_description_content_type = "text/markdown",
    url = "https://github.com/gitduk/tfuc.git",
    keywords=['selenium', 'bs4', 'web_craw'],
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'sqlalchemy', 'xlrd', 'xlsxwriter', 'openpyxl',
        'urllib3',
        'bs4', 'fake_useragent', 'selenium', 'requests',
    ],
)



import os
import sys
import time
from tools.timer import Time
import requests
from tools.logger import Logger
from tools.multithread import ThreadPool
import urllib3

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

lg = Logger()


class Downloader(object):
    def __init__(self, path, url, name, postfix='mp4'):
        self._path = path
        self._url = url
        self._name = name
        self._max_retry = 3
        self.postfix = postfix
        self.time = Time()
        self.time.set_fmt('%H:%M:%S')
        if not os.path.exists(path):
            os.makedirs(path)

    def __repr__(self):
        return '{}|{}|{}'.format(self._name, self.postfix, self._url)

    @property
    def path(self):
        return self._path

    @property
    def url(self):
        return self._url

    @property
    def name(self):
        return self._name

    @property
    def max_retry(self):
        return self._max_retry

    def run(self):
        max_retry_count = self.max_retry
        retry_count = 0
        while retry_count < max_retry_count:
            try:
                self.start_download()
                break
            except Exception as Ex:
                lg.error()
                retry_count += 1

        if retry_count >= max_retry_count:
            resp = requests.get(self._url)
            if resp.status_code != 404:
                time.sleep(60)
                try:
                    self.start_download()
                except:
                    lg.error()
            lg.warning("Download failed|{}".format(resp.status_code))

    @lg.log()
    def start_download(self):
        # deal with file type
        chunk_size = 1024
        self._name = self._name.strip().strip('.mp4')
        if self.postfix == 'img':
            time.sleep(1)
            chunk_size = 32
            self.postfix = self._url.split('.')[-1]

        # print('\n' + '-' * 100)
        start = time.time()
        _file_path = '{}/[✴]{}.{}'.format(self._path, self._name, self.postfix)
        file_path = '{}/{}.{}'.format(self._path, self._name, self.postfix)

        if os.path.exists(file_path):
            file_size = os.path.getsize(file_path)
            lg.info("File downloaded|{:.2f}MB|{}|{}".format(file_size / (1024 * 1024), self._name, self._url))
            return

        with requests.session() as req:
            req.stream = True
            req.verify = False
            req.timeout = 10
            req.headers = {
                "Accept-Encoding": "identity;q=1, *;q=0",
                "Range": None,
                "Referer": self._url,
                # "Connection": "Close",
                "User-Agent": "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/54.0.2840.59 Safari/537.36 115Browser/8.6.2"
            }
            # ================================================= first request get total length
            resp = req.get(self._url)
            is_chunked = resp.headers.get('transfer-encoding', '') == 'chunked'
            content_length_s = resp.headers.get('content-length')
            if not is_chunked and content_length_s.isdigit():
                total_length = int(content_length_s)
            else:
                total_length = None
                lg.warning('Content-Lenght is null|{}|{} |{}'.format(self._name, self._url, resp.status_code))
                return

            content_offset = 0
            if os.path.exists(_file_path):
                content_offset = os.path.getsize(_file_path)
                lg.info(
                    'Continue to download|{:.2f}MB/{:.2f}MB|{}|{}'.format(content_offset / (1024 * 1024),
                                                                          total_length / (1024 * 1024), self._name,
                                                                          self._url)
                )
            else:
                lg.info(
                    "Start to download|{:.2f}MB|{}|{}".format(total_length / (1024 * 1024), self._name, self._url))

            # =================================================== second request with head to download the remain data
            req.headers = {'Range': "bytes=%d-%d" % (content_offset, content_offset + total_length)}
            resp = req.get(self._url)
            size = content_offset
            fmt = ''
            pre_module = sys._getframe(1).f_code.co_filename.split('/')[-1].split('.')[0]
            with open(_file_path, 'ab') as file:
                progress_bar_length = 30
                for data in resp.iter_content(chunk_size=chunk_size):
                    file.write(data)
                    size += len(data)
                    done = int(progress_bar_length * size / total_length)
                    lg.info(
                        "[{}{}] [{:.2f}MB] [{:.2f}MB] [{}]".format('=' * done, ' ' * (progress_bar_length - done),
                                                                   size / (1024 * 1024),
                                                                   total_length / (1024 * 1024), self._name,
                                                                   # str(round(float(size / total_length) * 100, 2)), # %
                                                                   ), end='')
                    file.flush()
                os.rename(_file_path, file_path)
            end = time.time()
            lg.info(
                "Download successfully|{:.2f}S|{:.2f}MB|{}|{}".format(end - start, total_length / (1024 * 1024),
                                                                      self._name, self._url))

    @path.setter
    def path(self, value):
        self._path = value

    @url.setter
    def url(self, value):
        self._url = value

    @name.setter
    def name(self, value):
        self._name = value

    @max_retry.setter
    def max_retry(self, value):
        self._max_retry = value


class Download_from_file(object):
    def __init__(self, file_path, save_path, info=False):
        self.file_path = file_path
        self.save_path = save_path
        self.info = info
        self.selector = {'txt': '|', 'csv': ','}

    @property
    def downloader_list(self):
        project_list = []
        symbol = self.selector[self.file_path.split('.')[-1]]
        with open(self.file_path, 'r') as f:
            lines = f.readlines()
            if symbol == ',':
                lines = lines[1:]
            for i in lines:
                num = len(i.split(symbol))
                if num == 3:
                    url = i.split(symbol)[1]
                    name = i.split(symbol)[2]
                    d = Downloader(self.save_path, url, name, 'mp4')
                    project_list.append(d)
                elif num == 2:
                    url = i.split(symbol)[0]
                    name = i.split(symbol)[1]
                    d = Downloader(self.save_path, url, name, 'mp4')
                    project_list.append(d)
        return project_list

    def run(self, download_prject):
        download_prject.run()

    def start(self, max_thread=10):
        t_pool = ThreadPool(max_thread, info=self.info)
        key = self.file_path.split(".")[-1]
        t_pool.add_task_list(self.run, self.downloader_list)
        t_pool.execute_task()
